﻿using System;
using System.Windows;
using Microsoft.Phone.Controls;
using Windows.Networking.Proximity;

namespace _05_BTPeerFinder {
    public partial class MainPage : PhoneApplicationPage {
        public MainPage() {
            InitializeComponent();
            // Start advertising ourselves so that our peers can find us 
            PeerFinder.DisplayName = "05_BTPeerFinder";
            PeerFinder.Start();
        }

        /// <summary>
        /// Search for peer
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btnSearchPeer_Click(object sender, RoutedEventArgs e) {
            RefreshPeerAppList();
        }

        /// <summary> 
        /// Asynchronous call to re-populate the ListBox of peers. 
        /// </summary> 
        private async void RefreshPeerAppList() {
            try {
                WriteLog("finding peers ...");
                var peers = await PeerFinder.FindAllPeersAsync();
                if (peers.Count == 0) {
                    WriteLog("No Peers");
                } else {
                    WriteLog(String.Format("Fount {0} peers", peers.Count));
                    // Add peers to list 
                    foreach (var peer in peers) {
                        WriteLog(string.Format("Peer name: {0}", peer.DisplayName));
                    }
                }
            } catch (Exception ex) {
                WriteLog(string.Format("Error: {0}", ex.Message));
            }
        }

        /// <summary>
        /// Search for device
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private async void btnSearchDevice_Click(object sender, RoutedEventArgs e) {
            WriteLog("Finding devices...");
            PeerFinder.AlternateIdentities["Bluetooth:Paired"] = "";
            try {
                var peers = await PeerFinder.FindAllPeersAsync();
                foreach (PeerInformation peerInformation in peers) {
                    WriteLog(string.Format("Device {0}", peerInformation.DisplayName));
                    //StreamSocket _socket = await PeerFinder.ConnectAsync(peerInformation); 
                }
                // Handle the result of the FindAllPeersAsync call
            } catch (Exception ex) {
                if ((uint)ex.HResult == 0x8007048F) {
                    MessageBox.Show("Bluetooth is turned off");
                }
            }
        }

        /// <summary>
        /// Print log
        /// </summary>
        /// <param name="text"></param>
        private void WriteLog(string text) {
            Dispatcher.BeginInvoke(() => {
                    tbxLog.Text += text;
                    tbxLog.Text += "\r\n";
                });
        }
    }
}