﻿using System;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;
using NdefLibrary.Ndef;
using Windows.Networking.Proximity;

namespace _02_NDEF {
    public partial class MainPage : PhoneApplicationPage {
        // Constructor
        private ProximityDevice nfc;
        public MainPage() {
            InitializeComponent();
            nfc = ProximityDevice.GetDefault();
            nfc.DeviceArrived += nfc_DeviceArrived;
            nfc.DeviceDeparted += nfc_DeviceDeparted;
            nfc.SubscribeForMessage("NDEF", MessageReceivedHandler);
        }

        /// <summary>
        /// Message received
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="message"></param>
        private void MessageReceivedHandler(ProximityDevice sender, ProximityMessage message) {
            var rawMsg = message.Data.ToArray();
            var ndefMessage = NdefMessage.FromByteArray(rawMsg);
            // Loop over all records contained in the NDEF message
            foreach (NdefRecord record in ndefMessage) {
                Dispatcher.BeginInvoke(() =>
                    {
                        tbxLog.Text += String.Format("Record type: {0}\r\n", Encoding.UTF8.GetString(record.Type, 0, record.Type.Length));
                    });
                // Check the type of each record - handling a Smart Poster, URI and Text record in this example
                var specializedType = record.CheckSpecializedType(false);
                if (specializedType == typeof(NdefSpRecord)) {
                    // Convert and extract Smart Poster info
                    var spRecord = new NdefSpRecord(record);
                    Dispatcher.BeginInvoke(() =>
                    {
                        tbxLog.Text += string.Format("URI: {0}\r\n", spRecord.Uri);
                        tbxLog.Text += string.Format("Titles: {0}\r\n", spRecord.TitleCount());
                        tbxLog.Text += string.Format("1. Title: {0}\r\n", spRecord.Titles[0].Text);
                        tbxLog.Text += string.Format("Action set: {0}\r\n", spRecord.ActionInUse());

                    });
                    // You can also check the action (if in use by the record), 
                    //mime type and size of the linked content.
                } else if (specializedType == typeof(NdefUriRecord)) {
                    // Convert and extract URI info
                    var uriRecord = new NdefUriRecord(record);
                    Dispatcher.BeginInvoke(() =>
                    {
                        tbxLog.Text += string.Format("URI: {0}\r\n", uriRecord.Uri);
                    });
                } else if (specializedType == typeof(NdefTextRecord)) {
                    // Convert and extract Text record info
                    var textRecord = new NdefTextRecord(record);
                    Dispatcher.BeginInvoke(() =>
                    {
                        tbxLog.Text += string.Format("Text: {0}\r\n", textRecord.Text);
                        tbxLog.Text += string.Format("Language code: {0}\r\n", textRecord.LanguageCode);
                        var textEncoding = (textRecord.TextEncoding == NdefTextRecord.TextEncodingType.Utf8 ? "UTF-8" : "UTF-16");
                        tbxLog.Text += string.Format("Encoding: {0}\r\n", textEncoding);
                    });
                }
            }
        }

        /// <summary>
        /// Save text
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, RoutedEventArgs e) {
            string command = ((Button)sender).CommandParameter.ToString();
            switch (command) {
                case "text":
                    NdefTextRecord txtRecord = new NdefTextRecord();
                    txtRecord.Text = tbxText.Text;
                    NdefMessage txtMessage = new NdefMessage();
                    txtMessage.Add(txtRecord);
                    nfc.PublishBinaryMessage("NDEF:WriteTag", txtMessage.ToByteArray().AsBuffer(), WritingHandler);
                    break;
                case "url":
                    NdefUriRecord uriRecord = new NdefUriRecord();
                    uriRecord.Uri = "http://wpblog.michelelocuratolo.com";
                    NdefMessage uriMessage = new NdefMessage();
                    uriMessage.Add(uriRecord);
                    nfc.PublishBinaryMessage("NDEF:WriteTag", uriMessage.ToByteArray().AsBuffer(), WritingHandler);
                    break;
            }
        }

        /// <summary>
        /// Push message
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnPush_Click(object sender, RoutedEventArgs e) {
            string command = ((Button)sender).CommandParameter.ToString();
            switch (command) {
                case "text":
                    NdefTextRecord txtRecord = new NdefTextRecord();
                    txtRecord.Text = tbxText.Text;
                    NdefMessage txtMessage = new NdefMessage();
                    txtMessage.Add(txtRecord);
                    nfc.PublishBinaryMessage("NDEF", txtMessage.ToByteArray().AsBuffer());
                    break;
                case "url":
                    NdefUriRecord uriRecord = new NdefUriRecord();
                    uriRecord.Uri = "http://wpblog.michelelocuratolo.com";
                    NdefMessage uriMessage = new NdefMessage();
                    uriMessage.Add(uriRecord);
                    nfc.PublishBinaryMessage("NDEF", uriMessage.ToByteArray().AsBuffer());
                    break;
            }
        }

        /// <summary>
        /// Writing event 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="messageid"></param>
        private void WritingHandler(ProximityDevice sender, long messageid) {
            Dispatcher.BeginInvoke(() =>
            {
                nfc.StopPublishingMessage(messageid);
                tbxLog.Text += "Tag scritto\r\n";
                tbxLog.Text += string.Format("ID: {0}\r\n", messageid);
            });
        }


        /// <summary>
        /// Device arrived ebent
        /// </summary>
        /// <param name="sender"></param>
        void nfc_DeviceArrived(ProximityDevice sender) {
            Dispatcher.BeginInvoke(() =>
            {
                tbxProximityMessage.Text = "Device arrivato";
                tbxLog.Text = string.Empty;
                tbxLog.Text = " ------------\r\n";
            });
        }

        /// <summary>
        /// Device departed event
        /// </summary>
        /// <param name="sender"></param>
        void nfc_DeviceDeparted(ProximityDevice sender) {
            Dispatcher.BeginInvoke(() =>
            {
                tbxProximityMessage.Text = "Device allontanato";
            });
        }
    }
}